package edu.ubc.cpsc.googlex;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import org.json.JSONArray; // JSON library from http://www.json.org/java/
import org.json.JSONException;
import org.json.JSONObject;

/**
 * A simplified version of the GoogleSearch class.
 * <p>
 * Please use SimpleGoogleSearch for this assignment rather than using
 * GoogleSearch directly. However, feel free to read through the code for
 * SimpleGoogleSearch to understand GoogleSearch better. We do NOT recommend
 * reading through the code for LoggedGoogleSearch and LoggingGoogleSearch, as
 * they are designed for simplicity of use rather than readability. (We would
 * have designed them differently if we wanted them to be generally useful and
 * readable.)
 * <p>
 * SimpleGoogleSearch simplifies access to GoogleSearch in three ways. First, it
 * hides GoogleSearch's use of arrays (which we'll learn about shortly!).
 * Second, it hides GoogleSearch's use of exceptions (which we won't discuss in
 * 111, although 211 does discuss them). Third, it provides access to Logged and
 * Logging versions of GoogleSearch to ease debugging and marking. Note: we will
 * mark this assignment using the logged mode of SimpleGoogleSearch and the log
 * we provided you! You should ensure that your code works in that mode!
 * <p>
 * How do "logging" and "logged" modes work? Well, if both are disabled,
 * SimpleGoogleSearch actually goes to Google and searches for the queries you
 * pose to it. In "logged" mode, however, SimpleGoogleSearch does not connect to
 * the internet at all (which means you can use logged mode even without a
 * network connection). Instead, it uses a file called test.log that we've
 * distributed to you. That file contains a collection of stored searches.
 * "Logged" mode simply accesses those stored searches, printing an error
 * message if a stored search matching the search you requested cannot be found.
 * You don't really need to know about "logging" mode, but in case you want to
 * use it.. Logging mode works just like normal mode except any searches
 * performed in logging mode are appended to the file test.log for future
 * reference in logged mode. (We created test.log by running the sample solution
 * in logging mode and performing some searches!)
 */
public class SimpleGoogleSearch
{

    private final String HTTP_REFERER = "http://www.example.com/";

    /**
     * The response from Google in JSON format.
     */
    private JSONObject mySearchResponse;

    /**
     * The latest search result produced by a search.
     */
    private JSONArray myResult = null;
    
    /**
     * The number of results returned by the query
     */
    private int numResults = 0;

    /**
     * The number of hits returned by the query
     */
    private int hitCount = -1;

    
    /**
     * Default Constructor
     */
    public SimpleGoogleSearch ()
    {
        
    }
    
    /**
     * Search for the given query, that is, a string you might type into Google
     * when performing a search.
     * 
     * @param queryString a query (non-null)
     * @return true if the search succeeds (i.e., encounters no errors, even if
     *         it doesn't return any results) and false otherwise
     */
    public boolean doSearch( String queryString )
    {
        
        int index = 0;
        hitCount = -1;
        numResults = 0;

        try
        {
            myResult = new JSONArray();
            
            // Convert spaces to +, etc. to make a valid URL
            queryString = URLEncoder.encode(queryString, "UTF-8");
            
            for( int i = 0; i < 4; i++ )
            {
                
                URL url = new URL("http://ajax.googleapis.com/ajax/services/search/web?start=" + 8*i + "&rsz=large&v=1.0&q=" + queryString);
                URLConnection connection = url.openConnection();
                connection.addRequestProperty("Referer", HTTP_REFERER);
    
                // Get the JSON response
                String line;
                StringBuilder builder = new StringBuilder();
                BufferedReader reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
                while( ( line = reader.readLine() ) != null ) 
                {
                    builder.append(line);
                }
                String response = builder.toString();
                mySearchResponse = new JSONObject(response);

                JSONArray results = mySearchResponse.getJSONObject("responseData").getJSONArray("results");

                if( ! results.isNull( 0 ) )
                {
                    for (int j = 0; j < results.length(); j++)
                    {
                        if(!results.isNull( j ))
                        {
                            myResult.put( index, results.get( j ) );
                            index ++;
                            numResults ++;
                        }
                    }
                
                    if( hitCount == -1 )
                    {
                        try
                        {
                            hitCount = Integer.parseInt((mySearchResponse.getJSONObject("responseData").getJSONObject("cursor").getString("estimatedResultCount")).trim());
                        }
                        catch (Exception ex)
                        {
                            System.out.println("Error: Could not retrieve number of returned search results! Details: " + ex);
                        }
                    }
                }
                
            }
            return true;
        }
        catch (Exception e) 
        {
            System.out.println( "Search failed unexpectedly.  Error was: " + e );
            return false;
        }
    }


    /**
     * Get the number of hits the latest search produced.
     * 
     * @return the number of hits or -1 if there has not yet been a search or if
     *         the last search failed
     */
    public int getHitCount()
    {
        if( hitCount == -1 )
            return 0;
        
        return hitCount;
    }

    /**
     * Gets the number of search results (i.e., snippets) from the latest
     * search.
     * 
     * @return the number of search results or 0 if there has not yet been a
     *         search or if the last search failed
     */
    public int getNumResults()
    {
        return numResults;
    }

    /**
     * The snippet Google returned for the search result with the given index.
     * 
     * @param index a number between 0 and getNumResults - 1. NOTE THAT THIS
     *        STARTS AT ZERO!
     * @return the snippet at the given index
     */
    public String getSnippetAt( int index )
    {

        if( numResults == 0 )
            return null;
        
        JSONObject j;
        try
        {
            j = myResult.getJSONObject(index);
            return j.getString( "content" );
        }
        catch (JSONException ex)
        {
            System.out.println("Error: Could not parse response from Google! Details: " + ex);
            return "";
        }
    }
}
